"""
PISCES

Usage:
  main.py clearcaches
  main.py stats
  main.py [[map|export] [--usecache] [--notrans] [--export_png=<boolean>]]
  main.py [import [--test] [--testdir=<folder>]]
  main.py [help] [--help] [-h] [--usage]
"""

import os
import sys

try:
	from docopt import docopt  # by default, use the installed version
except:  # but if it's not installed, use a local copy
	from pisces_docopt import docopt

import local_vars
import args
import log
import funcs

#if __name__ == '__main__':
arguments = docopt(__doc__, version="1.x")
#set up the workspace
log.initialize()

if (arguments["--help"] or arguments["help"] or arguments["-h"] or arguments["--usage"]) \
	or (not (not (arguments["stats"] is False) or not (arguments["clearcaches"] is False)
		or not (arguments["import"] is False) or not
		(arguments["map"] is False) or not (arguments["export"] is False))):
	# TODO: WTF is this? Why? Isn't there a better way to do this with docopt?
	print __doc__
	sys.exit()

args.process_args(arguments)

if arguments["stats"]:
	t_cursor, t_conn = funcs.db_connect(local_vars.maindb, "getting stats")
	funcs.data_stats(t_cursor)
	funcs.db_close(t_cursor, t_conn)
	sys.exit()

if arguments["clearcaches"]:
	funcs.clean_workspace()
	funcs.refresh_layer_cache()
	sys.exit()

if arguments["--notrans"]:
	local_vars.current_obs_types = local_vars.notrans_obs_types

local_vars.print_workspace_vars()
local_vars.data_setup()  # populates variables with data from the database
#funcs.clean_workspace() #make sure that old data files are cleared out


# run stats for a normal run here so that they sit for a while while everything else gets set up
t_cursor, t_conn = funcs.db_connect(local_vars.maindb, "getting stats")
funcs.data_stats(t_cursor)
funcs.db_close(t_cursor, t_conn)


### Process Any New Data ###

if arguments["import"]:
	if arguments["--test"]:
		args.flags["test"] = 1
		funcs.setup_test_mode()

	if arguments["--testdir"]:
		if arguments["--testdir"] == "temp":
			import tempfile

			local_vars.test_folder = os.path.join(tempfile.gettempdir(), "pisces")
		else:
			local_vars.test_folder = arguments["--testdir"]

	print "BEGINNING NEW DATA PROCESSING"

	funcs.new_data_fetch()  # checks for new data, validates that it exists, and puts the info in the right spots
	funcs.setup_new_data(local_vars.datasets)

	#  tried to implement multiprocessing here, but due to ESRI file locks, it will probably require significant
	#  refactoring of input filters and how they hold data to work, if it's possible. Not worth it now. May be someday,
	#  but unlikely. Sad face... Would have been pretty cool to have this thing scream through some datasets!

	for l_set in local_vars.datasets:  # now that the datasets are loaded process them!
		if l_set is not None:  # it's possible we've zeroed it out if it was an invalid dataset
			l_set.process_data()  # will call the input filter for each dataset to handle the importing, then call cleanups

	db_cursor, db_conn = funcs.db_connect(local_vars.maindb, "Inserting results!")  # open the db!

	log.write("\nInserting new records!", 1)

	for l_set in local_vars.datasets:
		if l_set is not None:
		# TODO: Handle DataStorageErrors when thrown - we'll need to roll back a lot of things
		#  depending upon where they were thrown (call dataset.cleanup(), then zero it out)
			l_set.record_data(db_cursor)

	funcs.db_close(db_cursor, db_conn)  # close that sucker

if arguments["map"] or arguments["export"]:
	if arguments["--usecache"]:
		local_vars.usecache = 1

	import mapping

	try:
		mapping.begin("all")
	except local_vars.MappingError as error:
		print "Uncaught error encountered while mapping - program provided: %s" % error

print "\nComplete"