import os
import sys
import re

import arcpy

import local_vars
import funcs
import log

from code_library.common.utils2 import semicolon_split_to_array  # some code references this here. Should be careful with it.

''' set of functions meant for use in ArcGIS Script Tools '''

'''def run_query(l_query,db_conn,bind_vars):'''
'''Given a query from a script tool, this runs it, and returns the results, if any. To be used as the base of other tools'''
'''
	db_cursor = db_conn.cursor()

	if len(bind_vars) > 0:
		l_results = db

	pass'''


def get_location():

	try:
		import _winreg
		registry = _winreg.ConnectRegistry("",_winreg.HKEY_CURRENT_USER) # open the registry
		base_folder = _winreg.QueryValue(registry,"Software\CWS\PISCES\location") # get the PISCES location
		_winreg.CloseKey(registry)
	except:
		log.error("Unable to get base folder")
		sys.exit()

	if local_vars.internal_workspace != base_folder:  # essentially, if we haven't already run this
		local_vars.set_workspace_vars(base_folder)  # set up the workspace to the location

	return base_folder


def get_fish_filter():
	db_cursor, db_conn = funcs.db_connect(local_vars.maindb)
	get_fish = "select FID,Common_Name from Species order by Common_Name asc"

	l_fish = []

	fishies = db_cursor.execute(get_fish)
	for fish in fishies:
		l_str = "%s - %s" % (fish.FID,fish.Common_Name)
		l_fish.append(l_str)

	funcs.db_close(db_cursor, db_conn)
	return l_fish


def make_species_group_picker(validation_class, selector_index, storage_index):
	"""

	:param validation_class: the arcmap tool validation class objet
	:param selector_index:
	:param storage_index:
	:return:
	"""
	validation_class.selected_species = ""
	validation_class.blank = ""

	try:
		l_fish = get_fish_and_groups_filter(prepend_blank=True, sep="-----")  # add a blank item to the beginning
	except:
		l_fish = []

	if len(validation_class.params) > 1:  # stupid arcgis fix because it won't validate otherwise
		validation_class.params[selector_index].filter.list = l_fish
		validation_class.params[selector_index].value = validation_class.blank


def add_selection(val_class, selector_index, storage_index):

	single_species = autocomplete_full_field(val_class, selector_index)

	# move species lookups to storage field
	if single_species: # if we actually found a species in our autocomplete - thus can add it to the list
		val_class.selected_species = str(val_class.params[storage_index].value)
		if val_class.selected_species == "None":  # starts out this way
			val_class.selected_species = val_class.params[selector_index].value # so replace it
		else:
			val_class.selected_species += ";%s" % val_class.params[selector_index].value # otherwise append it
		val_class.selected_species.replace("None;", "")  # it tends to get placed in the beginning
		val_class.params[storage_index].value = val_class.selected_species  # assign the string to the field
		val_class.params[selector_index].value = val_class.blank  # set the species selector to blank
	#else:  # we didn't find a species via autocomplete - but it's still possible that we manually selected a species
	#	if val_class.params[selector_index].value in val_class.params[storage_index].value:
	#		val_class.params[selector_index].value = val_class.blank # set the species selector to blank


def get_groups_filter():
	db_cursor, db_conn = funcs.db_connect(local_vars.maindb)
	get_group = "select Group_Name from defs_Species_Groups order by Group_Name asc"

	l_group = []

	groupies = db_cursor.execute(get_group)
	for group in groupies:
		l_group.append(group.Group_Name)

	funcs.db_close(db_cursor, db_conn)
	return l_group


def get_fish_and_groups_filter(prepend_blank=True, sep = "----"):
	"""

	:param prepend_blank: Boolean. Adds a " " at the beginning to be selectable
	:param sep: a separator between the groups and the fish
	:return:
	"""

	return_list = []
	if prepend_blank:
		return_list.append(" ")

	return_list += get_groups_filter()

	if sep:  # append the separator
		return_list.append(sep)

	return_list += get_fish_filter()

	return return_list


def validate_species(field):

	if field.hasBeenValidated:
		return

	if field.value and (str(field.value.lower()) == "load" or str(field.value.lower()) == "last"):
		trans = load_transaction("last")
		field.value = trans['Species_In']


def load_transaction(l_id="last",db_cursor = None):

	opened_db = False
	if db_cursor is None:
		opened_db = True

		db_cursor, db_conn = funcs.db_connect(local_vars.maindb, "loading transactions")

	results = None
	if l_id == "last":
		sql = "select * from Transactions order by ID desc"
		results = db_cursor.execute(sql)
	else:
		sql = "select * from Transactions where ID = ?"
		results = db_cursor.execute(sql)

	t = results.fetchone()

	transaction = {}
	if t:
		transaction['Species_In'] = t.Species_In
		transaction['Message'] = t.Message
		transaction['Species_To'] = t.Species_To
		transaction['Operation'] = t.Operation
	else:
		transaction['Species_In'] = ""
		transaction['Message'] = ""
		transaction['Species_To'] = ""
		transaction['Operation'] = ""

	if opened_db:
		funcs.db_close(db_cursor, db_conn)

	return transaction


def autocomplete_full_field(obj, field_num=1):  # given a script tool field, it will search it for what you typed to autocomplete
	"""

	:param obj: the validation object from arcgis
	:param field_num: the field to validate
	:return: boolean indicating whether a single, valid species is currently selected
	"""

	min_length = 5  # set the min length for trying a subset match

	if obj.params[field_num].hasBeenValidated:  # if this field hasn't changed since the last time we validated it
		return

	# check if it's already in the list - ie, we selected from the list
	for species in obj.params[field_num].filter.list:
		if obj.params[field_num].value == species:
			return True

	# if we're still here, it was probably typed - if it was perfectly typed it would be caught above, so check for partials
	if obj.params[field_num].value and len(obj.params[field_num].value) < min_length:  # if it's short, don't autoselect - not specific enough
		return False

	t_count = 0
	ts = None
	for species in obj.params[field_num].filter.list:
		if str(species).lower().find(str(obj.params[field_num].value).lower()) > -1:  # if what they typed is in this species - lowercase matching
			ts = species
			t_count += 1

	if t_count > 1:
		obj.params[field_num].value = "%s items found..." % t_count
		return False
	if t_count == 1:
		obj.params[field_num].value = ts
		return True


def load_map_sets():
	"""
	Loads the currently available maps in the database - just the attributes Set_Name, Map_Title, and Set_Description.
	Returns a list of objects with those attributes.

	:return: list of map objects, descriptive map string list
	"""

	class t_map:
		def __init__(self, name, title, desc):
			self.Set_Name = name
			self.Map_Title = title
			self.Set_Description = desc
			self.string_rep = name

	db_cursor, db_conn = funcs.db_connect(local_vars.maindb, "Loading maps")

	map_query = "Select Set_Name, Map_Title, Set_Description from defs_Query_Sets order by Set_Name asc"

	results = db_cursor.execute(map_query)

	last_desc = "Runs the last map configuration that was set. If you just ran this tool it will output the same set"

	last_map = t_map("Last Map Configuration", "", last_desc)

	maps = [last_map, ]
	map_strings = ["Last Map Configuration", ]
	for result in results:

		l_map = t_map(result.Set_Name, result.Map_Title, result.Set_Description)

		maps.append(l_map)
		map_strings.append(l_map.string_rep)

	funcs.db_close(db_cursor, db_conn)

	return maps, map_strings


def zones_feature_to_array(zones_layer):
	l_cursor = arcpy.SearchCursor(zones_layer)

	zones_array = []

	for feature in l_cursor: # should only iterate over the selected features
		zones_array.append(feature.HUC_12) # TODO: Fix HUC_12 Hardcode 

	return zones_array


def query_datasets_by_zone(zones, db_conn, species=None):
	# what does this function do??

	db_cursor = db_conn.cursor()

	if not (type(zones) is tuple or type(zones) is list):
		zones = [zones] # make it into a list so that the same code works

	species_ext = ""	
	if species is not None:
		species_ext = " and t1.Species_ID = '%s'" % species

	query = "select distinct t1.Set_ID, t2.Source_Data, t2.Input_Filter, t1.Presence_Type from Observations as t1, Observation_Sets as t2 where t1.Zone_ID = ? and t1.Set_ID = t2.Set_ID%s" % species_ext
	arcpy.AddMessage("Running query - %s" % query)
	observation_sets = {}
	hucs = {}
	tablerows_by_source = []

	#TODO: Fix this block - it's probably broken
	for zone in zones:
		l_results = db_cursor.execute(query, zone)
		for result in l_results:
			if not (result.Set_ID in observation_sets):
				observation_sets[result.Set_ID] = local_vars.observation_set()
				observation_sets[result.Set_ID].zones = []  # initialize an empty array of zones
				observation_sets[result.Set_ID].dataset_path = result.Source_Data  # set the attributes
				observation_sets[result.Set_ID].Filter_Code = result.Input_Filter

			tablerows_by_source.append(html_add_tablerow([result.Set_ID, result.Input_Filter, result.Source_Data, zone, result.Presence_Type]))  # store a table row to print to the final file

			arcpy.AddMessage("%s - %s - %s - %s - %s" % (result.Set_ID, result.Input_Filter, result.Source_Data, zone, result.Presence_Type))  # and print it directly to the arcpy console

	import os	
	l_out_filename = os.path.join(local_vars.internal_workspace, "log", "query_datasets_by_zone_out.htm")

	if os.path.exists(l_out_filename):
		os.remove(l_out_filename)  # get rid of the old file

	zone_out = open(l_out_filename, 'w')  # and open a new one

	zone_out.write('<html><head><title>' + species + ' Source information</title>'
				   '<style>body{font-family:Arial,Helvetica, sans-serif;color:#666;}</style></head>'
				   '<body><h1>' + species + ' Source information</h1><table cellspacing="10">'
				   '<tr><td>Observation Set ID</td><td>Input Filter Code</td><td>Dataset Path</td>'
				   '<td>Zone ID</td><td>Presence Type</td>')  # make a rudimentary non-standards-compliant html page

	for row in tablerows_by_source:
		zone_out.write(row)

	zone_out.write("</table></body></html>")

	arcpy.AddWarning("Full output is located at %s" % l_out_filename)

	#TODO: Make display better and highlight/note HUCs with multiple sources

	db_cursor.close()


def parse_multi_species_group_list(l_species_and_groups):

	listing = l_species_and_groups.split(";")
	new_listing = []
	for item in listing:
		if "-" in item:  # if we have a separator - this is clunky, but the best way
							#  I can think of since we're parsing species and groups
			new_item = parse_input_species_from_list(item)
		else:
			new_item = item

		if item != '':  # we might get some empty items from the split
			new_listing.append(new_item)

	return new_listing


def parse_input_species_from_list(l_search_species):
	"""
	takes a species from an ArcGIS 10 Script tool dropdown in the form of "Code - Common Name" and parses out the code and returns it
	:param str l_search_species: species listing from script tool
	:return: str FID
	"""
	species_re = re.search("^'?(\w{3}\d{2})", l_search_species)
	try:
		return species_re.group(1)
	except:
		return None  # if it didn't work, we probably don't have a species


def parse_obs_type_from_list(l_search_string):
	'''takes an observation code from an ArcGIS 10 Script tool dropdown in the form of "Code - Description" and parses out the code and returns it'''

	type_re = re.search("^'?(\d+)", l_search_string)
	try:
		return int(type_re.group(1))
	except:
		return None  # if it didn't work, we probably don't have an obs type


def html_add_tablerow(items):
	l_row = "<tr>\n"
	for item in items:
		l_row = "%s<td>%s</td>\n" % (l_row,item) # add a table cell for the item
	l_row = "%s\n</tr>" % l_row # end the row

	return l_row
