__author__ = 'Nick'

import os

import arcpy

from PISCES import local_vars
from PISCES import funcs
from PISCES import script_tool_funcs
from PISCES import log
from PISCES import mapping

script_tool_funcs.get_location()
log.initialize(arc_script=True)
local_vars.data_setup()

map_name = arcpy.GetParameterAsText(0)
# 1 is a description holder and 2 is the selection for the groups
config_species_and_groups_list = arcpy.GetParameterAsText(3)
config_auto_open = arcpy.GetParameter(4)

config_output_ddp = arcpy.GetParameter(5)
config_output_png = arcpy.GetParameter(6)
config_output_pdf = arcpy.GetParameter(7)
config_output_kml = arcpy.GetParameter(8)
config_output_shp = arcpy.GetParameter(9)

if map_name != "Last Map Configuration":  # If we haven't specified to keep everything the way it is.
	# takes the string and separates it out into individual items,
	#  extracting the species from their "code - common name" form
	config_species_list = script_tool_funcs.parse_multi_species_group_list(config_species_and_groups_list)

	# connect to the DB
	db_cursor, db_conn = funcs.db_connect(local_vars.maindb, "exporting maps from script tool")

	# disable all maps
	disable_maps_query = "Update %s set Active=No" % local_vars.maps_table
	db_cursor.execute(disable_maps_query)

	# enable the relevant one
	enable_correct_map_query = "Update %s set Active=Yes where Set_Name='%s'" % (local_vars.maps_table, map_name)
	db_cursor.execute(enable_correct_map_query)
	db_conn.commit()  # commit the changes

	# get the map id
	map_id_query = "select ID from %s where Set_Name='%s'" % (local_vars.maps_table, map_name)
	map_id = db_cursor.execute(map_id_query).fetchone().ID

	log.write("Map ID: %s" % map_id, 1)
	if not map_id:
		log.error("Couldn't find Map ID for selected map")
		raise ValueError("Couldn't find Map ID for selected map")

	# set the bind values, regardless of if the map takes them

	# delete all existing bind for the map
	remove_bind_query = "delete from %s where Query_Set_ID = %s" % (local_vars.bind_vals_table, map_id)
	db_cursor.execute(remove_bind_query)

	# insert the new species records
	insert_bind_query = "insert into %s (Query_Set_ID,Bind_Value) values (%s,?)" % (local_vars.bind_vals_table, map_id)
	for species in config_species_list:
		db_cursor.execute(insert_bind_query, species)

	# commit the changes again
	db_conn.commit()
	funcs.db_close(db_cursor, db_conn)

# set some mapping variables
mapping.export_mxd = True

mapping.export_pdf = config_output_pdf
mapping.export_png = config_output_png
mapping.export_ddp = config_output_ddp
mapping.export_web_layer_kml = config_output_kml  # setting either of these two to True will greatly increase processing time
mapping.export_web_layer_shp = config_output_shp

# execute the map
map_objects = mapping.begin("all", return_maps=True)  # run the maps and get the objects

if config_auto_open:  # if we should auto-open it
	if map_objects and len(map_objects) == 1:  # if we have just one map, try to auto-open
		l_map = map_objects[0]  # the resulting map

		log.write("Opening MXD", 1)
		os.startfile(l_map.mxd_path)  # subprocess.call([os.path.join(install_dir, "Bin", "ArcMap.exe"), ]) # call arcmap with the mxd
	else:  # otherwise, it's too many maps - don't want to open 20 maps, etc
		log.error("Too many maps to open - please open them manually out of the mxds/output directory of your PISCES"
					"install folder")