__author__ = 'nickrsan'

import unittest

import arcpy

from PISCES import funcs
from PISCES import local_vars



class ComposeQueryTest(unittest.TestCase):

	def setup(self):
		self.db_cursor, self.db_conn = funcs.db_connect(local_vars.maindb, "Unit Test - 'test_funcs.py'")

	def test_compose_query(self):
		fields = {'a': 'b', 'c': 'd'}
		table = "test"
		query_result = funcs.compose_query_from_dict(table, value_pairs=fields)

		self.assertEqual("INSERT INTO test (a, c) VALUES ('b', 'd')", query_result)

		fields['e'] = 5
		fields['f'] = 5.123

		query_result2 = funcs.compose_query_from_dict(table, value_pairs=fields)
		self.assertEqual("INSERT INTO test (a, c, e, f) VALUES ('b', 'd', 5, 5.123)", query_result2)

		fields['g'] = None
		fields['h'] = 31.53214312
		fields['i'] = None
		fields['jj'] = "Peter Moyle says they're here"

		query_result3 = funcs.compose_query_from_dict(table, value_pairs=fields)
		self.assertEqual("INSERT INTO test (a, c, e, g, f, i, h, jj) VALUES ('b', 'd', 5, NULL, 5.123, NULL, 31.53214312, 'Peter Moyle says they''re here')", query_result3)

	def test_query_to_dicts(self):
		self.setup()

		# testing that it appropriately drops "OBJECTID"
		records = funcs.query_to_list_of_dicts("SELECT * FROM Observations WHERE Zone_ID = '180200010404'", self.db_cursor, drop_fields=["OBJECTID",])

		self.assertNotIn("objectid", records[0])  # assert that there is no OBJECTID field

		# testing that it appropriately keeps "OBJECTID"
		records = funcs.query_to_list_of_dicts("SELECT * FROM Observations WHERE Zone_ID = '180200010404'", self.db_cursor)

		self.assertIn("objectid", records[0])

	def test_refresh_layer_cache(self):
		"""
			incomplete testing. Just tests to make sure the gdb exists afterward. Doesn't check writes.
		"""

		funcs.refresh_layer_cache()
		self.assertTrue(arcpy.Exists(local_vars.layer_cache))
		## TODO: This should check that there are no records in the layer cache table

	def test_create_collection(self):
		self.setup()

		results = funcs.run_query("select max(ID) as max_id from defs_Collections")
		should_be_id = results[2].fetchone().max_id + 1
		collection_id = funcs.create_collection("Test", "test", "Testing insertion. Should not stay because it won't be committed", self.db_cursor)
		self.assertEqual(should_be_id, collection_id)




if __name__ == '__main__':
	unittest.main()
